# The software in this package is distributed under the GNU General
# Public License version 2 (with a special exception described below).
#
# A copy of GNU General Public License (GPL) is included in this distribution,
# in the file COPYING.GPL.
#
# As a special exception, if other files instantiate templates or use macros
# or inline functions from this file, or you compile this file and link it
# with other works to produce a work based on this file, this file
# does not by itself cause the resulting work to be covered
# by the GNU General Public License.
#
# However the source code for this file must still be made available
# in accordance with section (3) of the GNU General Public License.
#
# This exception does not invalidate any other reasons why a work based
# on this file might be covered by the GNU General Public License.

""" Unittest for follow.py

This is a little more involved since in order to test properly, need a second
thread that writes to file/socket/stdout

.. todo:: as in log_read_unittest: disable buffering in LogFileWriter

.. warn:: DEPRECATED (at least for files see log_read; may still be useful for
          pipes/sockets)

.. codeauthor:: Intra2net
"""


import unittest
from tempfile import mkstemp
import threading
from follow import *
from time import sleep
import os
import os.path
from test_helpers import get_perf_counter

# different counter needed on different python versions
perf_counter = get_perf_counter()


class LogFileWriter(threading.Thread):
    """ thread that creates and writes to given file handle """

    def __init__(self, file_name, text_pattern, n_writes=None,
                 pause_time=0.1, do_encode=None):
        """ creates thread, deamon is True
        
        if n_writes is None, will write indefinitely; else writes text_pattern
        n_writes times, formatted with (counter, perf_counter)
        """
        super(LogFileWriter, self).__init__()
        self.daemon = True
        self.file_name = file_name
        self.text_pattern = text_pattern
        self.n_writes = n_writes
        self.pause_time = pause_time
        self.do_encode = do_encode

    def run(self):
        counter = 0
        if self.do_encode:
            mode = 'wb'
        else:
            mode = 'wt'

        with open(self.file_name, mode) as file_handle:
            while True:
                if self.n_writes is not None and counter >= self.n_writes:
                    break

                if self.do_encode:
                    file_handle.write(self.text_pattern
                                           .format(counter, perf_counter())
                                           .encode(self.do_encode))
                else:
                    file_handle.write(self.text_pattern
                                           .format(counter, perf_counter()))
                print('wrote {0}'.format(counter))
                counter += 1
                sleep(self.pause_time)


@unittest.skip("Need to debug but deprecated anyway")
class FollowTester(unittest.TestCase):
    """ Unittest for follow.py """

    def test_logfile(self):
        """ create logfile, write to it, expect output """

        text_pattern = '{0}:{1}\n'
        n_texts = 10
        pause_time = 1
        encoding = 'ascii'

        LogFileWriter(self.temp_file, text_pattern, n_writes=n_texts,
                      pause_time=pause_time, do_encode=encoding).start()
        print('testing with log file {0}'.format(self.temp_file))
        time_diffs = []

        with open(self.temp_file) as read_handle:
            follower = Follower(read_handle)
            for counter, (source, desc, text, flags) in \
                    enumerate(follower):
                receive_time = perf_counter()
                print('received text: "{0}"'.format(text))
                index = text.index(':')
                counter = int(text[:index].strip())
                write_time = float(text[index+1:].strip())
                time_diffs.append(receive_time - write_time)
                if counter == n_texts:
                    break


    def setUp(self):
        """ called before each test """
        print('setup test')
        temp_handle, temp_name = mkstemp()
        os.close(temp_handle)
        self.temp_file = temp_name
        print('created temp file ' + self.temp_file)

    def tearDown(self):
        """ called after each test """
        print('tear down test')
        if os.path.isfile(self.temp_file):
            print('delete temp file' + self.temp_file)
            os.unlink(self.temp_file)

    @classmethod
    def setUpClass(clz):
        """ called once before the first test """
        print('setup test class')
        clz.temp_file = None

    @classmethod
    def tearDownClass(clz):
        """ called once after the last test """
        print('tear down test class')



if __name__ == '__main__':
    unittest.main()
